#include "stdafx.h"

#include "config.h"

/**************************
This tutorial is meant to complement to normal SDK documentation,
it does not replace it. In particular, you should read the SDK readme
before or while studying this tutorial.
**************************/

/**************************
Providing information about a component.

We can provide some information about our component that 
users will be able to view under Preferences > Components.
**************************/


DECLARE_COMPONENT_VERSION(
	// component name
	"foo_stopaftercuralbum",
	// component version
	"0.2",
	// about text, use \n to separate multiple lines
	// If you don't want any about text, set this parameter to NULL.
	NULL
);




/**************************
Component initialization and shutdown.

Global (static) variables can be assigned an initial value
when loading the DLL by adding an initializer to their
declarations. However, configuration variables don't "know"
their value at that point of time and services aren't available
yet.

On the other hand, a component may need to perform some kind of
cleanup when the application is shutting down. Doing cleanup
when the DLL is unloading may be too late, since services are no
longer available. One example would be storing the final value
of some setting (window position!) in a configuration variable.

So a properly behaving should component should fully initialze
itself when all components have been loaded, and should perform
any necessary cleanup before any component is unloaded. For this
purpose exists the initquit callback.
**************************/

#if 0
// Our component will show its window at initialization time, if
// the window is enabled.

class initquit_tutorial1 : public initquit {
	virtual void on_init() {
		// Show the window, if it is enabled.
		if (cfg_enabled)
			CTutorialWindow::ShowWindow();
	}

	virtual void on_quit() {
		// Do nothing.
		// The window placement variable will automatically store the 
		// position and size of our window, if it is currently visible.
	}
};

static initquit_factory_t< initquit_tutorial1 > foo_initquit;

#endif



/**************************
Providing menu commands.

Menu commands are one of the most important means for users
to interact with a general purpose component. (You may argue
that a component may have its own window with that the user
interacts most of the time, but how does the user open this
window? See?)

There are two basic types of menu commands: main menu and
context menu commands. This tutorial only uses main menu
commands.
**************************/

class mainmenu_commands_stopaftercuralbum : public mainmenu_commands
{

public:

	// Return the number of commands we provide.
	virtual t_uint32 get_command_count() {
		return 1;
	}

	// All commands are identified by a GUID.
	virtual GUID get_command(t_uint32 p_index) {
        // {00B9369D-9AAE-4ccb-B24F-1A92C46D237E}
        static const GUID guid_main_stopaftercuralbum_toggle = { 0xb9369d, 0x9aae, 0x4ccb, { 0xb2, 0x4f, 0x1a, 0x92, 0xc4, 0x6d, 0x23, 0x7e } };

		if (p_index == 0)
			return guid_main_stopaftercuralbum_toggle;
		return pfc::guid_null;
	}

	// Set p_out to the name of the n-th command.
	// This name is used to identify the command and determines
	// the default position of the command in the menu.
	virtual void get_name(t_uint32 p_index, pfc::string_base & p_out) {
		if (p_index == 0)
			p_out = "Stop After Current Album";
	}

	// Set p_out to the description for the n-th command.
	virtual bool get_description(t_uint32 p_index, pfc::string_base & p_out) {
		if (p_index == 0)
			p_out = "Stops playback after currently played album.";
		else
			return false;
		return true;
	}

	// Every set of commands needs to declare which group it belongs to.
	virtual GUID get_parent()
	{
        return mainmenu_groups::playback;
	}

	// Execute n-th command.
	// p_callback is reserved for future use.
	virtual void execute(t_uint32 p_index, service_ptr_t<service_base> p_callback) {
		if (p_index == 0 && core_api::assert_main_thread()) {
        	cfg_enabled = !cfg_enabled;
		}
	}

	// The standard version of this command does not support checked or disabled
	// commands, so we use our own version.
	virtual bool get_display(t_uint32 p_index, pfc::string_base & p_text, t_uint32 & p_flags)
	{
		p_flags = 0;
		if (is_checked(p_index))
			p_flags |= flag_checked;
		get_name(p_index,p_text);
		return true;
	}

//	virtual t_uint32 get_sort_priority() {return sort_priority_base;}

private:

	// Return whether the n-th command is checked.
	bool is_checked(t_uint32 p_index) {
		if (p_index == 0)
			return cfg_enabled;
		return false;
	}

};

// We need to create a service factory for our menu item class,
// otherwise the menu commands won't be known to the system.
static mainmenu_commands_factory_t<mainmenu_commands_stopaftercuralbum> g_mainmenu_commands_stopaftercuralbum;


class stop_playback_callback : public main_thread_callback
{

public:

    virtual void callback_run( void )
    {
        static_api_ptr_t<playback_control> playback_control_api;
        playback_control_api->stop();
    }

};


class play_callback_stopaftercuralbum : public play_callback_static
{

public:

    play_callback_stopaftercuralbum( void )
    {
        m_last_played_id_string = "";
    }

    virtual unsigned get_flags( void )
    {
        return( flag_on_playback_new_track | flag_on_playback_stop );
    }

	virtual void FB2KAPI on_playback_new_track( metadb_handle_ptr p_track )
    {
        pfc::string8 new_id_string;

        p_track->metadb_lock();

        const file_info* p_fileinfo;
        if( p_track->get_info_locked(p_fileinfo) )
        {
            // Fetch album ID format string and evaluate it with the
            // titleformatting stuff.
            pfc::string8 id_format;
            cfg_album_pattern.get( id_format );
            p_track->format_title_legacy( NULL, new_id_string, id_format, NULL );

// Old hard-coded approach (v0.1).
#if 0
            const char* album_artist = p_fileinfo->meta_get( "ALBUM ARTIST", 0 );
            const char* album        = p_fileinfo->meta_get( "ALBUM", 0 );
            const char* date         = p_fileinfo->meta_get( "DATE", 0 );

            new_id_string += ( album_artist ? album_artist : "" );
            new_id_string += " | ";
            new_id_string += ( album ? album : "" );
            new_id_string += " | ";
            new_id_string += ( date ? date : "" );
            new_id_string += " | ";
            new_id_string += pfc::io::path::getDirectory(p_track->get_path()).get_ptr();
#endif


            #if _DEBUG
            console::info( "foo_stopaftercuralbum: new id string is:" );
            console::info( new_id_string.get_ptr() );
            #endif
        }

        p_track->metadb_unlock();

        bool need_to_stop = false;

        // When this ID string changes we will regard the album as having
        // changed.
        if( !new_id_string.is_empty() )
        {
            if( m_last_played_id_string.is_empty() )
            {
                // This is the first track played after loading the player or
                // stopping playback previously. It may have been us that
                // stopped playback, of course. Just store the new string for
                // next time.
                m_last_played_id_string = new_id_string;

                #if _DEBUG
                console::info( "foo_stopaftercuralbum: playing a new album - just storing id string" );
                #endif
            }
            else
            {
                // Compare against last track played.
                if( pfc::stricmp_ascii(new_id_string.get_ptr(), m_last_played_id_string.get_ptr()) )
                {
                    if( cfg_enabled )
                    {
                        need_to_stop = true;
                    }
                    else
                    {
                        #if _DEBUG
                        console::info( "foo_stopaftercuralbum: album changed but option currently disabled" );
                        #endif
                    }
                }
                else
                {
                    #if _DEBUG
                    console::info( "foo_stopaftercuralbum: same album! allowing playback to continue" );
                    #endif
                }
            }
        }

        if( need_to_stop )
        {
            console::info( "foo_stopaftercuralbum: Stopping playback because album has changed." );

            // Queue up a callback to stop playback. (It sounds like it's
            // unsafe to do it directly from this callback.)
            static_api_ptr_t<main_thread_callback_manager> callback_manager_api;
            callback_manager_api->add_callback( new service_impl_t<stop_playback_callback>() );

            // If the user has enabled the 'only stop once' option then
            // disable ourselves.
            if( cfg_only_stop_once )
            {
                cfg_enabled = false;
            }

            // NB: We'll deal with clearing our cached ID string in the
            // stop callback (since we'll also have to deal with the same
            // thing if the user stops playback manually).
        }
    }

	virtual void on_playback_stop( play_control::t_stop_reason p_reason )
    {
        #if _DEBUG
        console::info( "foo_stopaftercuralbum: playback has stopped; clearing last id string" );
        #endif

        // Blank out the last ID for next time.
        m_last_played_id_string = "";
    }

	virtual void on_playback_starting(play_control::t_track_command p_command,bool p_paused) {}
	virtual void on_playback_seek(double p_time) {}
	virtual void on_playback_pause(bool p_state) {}
	virtual void on_playback_edited(metadb_handle_ptr p_track) {}
	virtual void on_playback_dynamic_info(const file_info & p_info) {}
	virtual void on_playback_dynamic_info_track(const file_info & p_info) {}
	virtual void on_playback_time(double p_time) {}
	virtual void on_volume_change(float p_new_val) {}

private:

    pfc::string8 m_last_played_id_string;

};

static play_callback_static_factory_t<play_callback_stopaftercuralbum> g_play_callback_stopaftercuralbum;